/* Copyright 2000  AG Electronics Ltd. */
/* Copyright 2003-2004 Linux Networx */
/* Copyright 2004 Tyan 
   By LYH change from PC87360 */
/* This code is distributed without warranty under the GPL v2 (see COPYING) */

#include <arch/io.h>
#include <device/device.h>
#include <device/pnp.h>
#include <console/console.h>
#include <string.h>
#include <bitops.h>
#include <uart8250.h>
#include <pc80/keyboard.h>
#include <pc80/mc146818rtc.h>
#include "chip.h"
#include "it8716f.h"


static void pnp_enter_ext_func_mode(device_t dev) 
{
	uint8_t last = 0x55;
        outb(0x87, dev->path.u.pnp.port);
        outb(0x01, dev->path.u.pnp.port);
        outb(0x55, dev->path.u.pnp.port);
	if(dev->path.u.pnp.port == 0x4e) {
		last <<= 1;
	}
        outb(last, dev->path.u.pnp.port);
}
static void pnp_exit_ext_func_mode(device_t dev) 
{
	pnp_write_config(dev, 0x02, 0x02);
}

static void it8716f_init(device_t dev)
{
	struct superio_ite_it8716f_config *conf;
	struct resource *res0, *res1;
	if (!dev->enabled) {
		return;
	}
	conf = dev->chip_info;
	switch(dev->path.u.pnp.device) {
	case IT8716F_SP1: 
		res0 = find_resource(dev, PNP_IDX_IO0);
		init_uart8250(res0->base, &conf->com1);
		break;
	case IT8716F_SP2:
		res0 = find_resource(dev, PNP_IDX_IO0);
		init_uart8250(res0->base, &conf->com2);
		break;
	case IT8716F_KBCK:
		res0 = find_resource(dev, PNP_IDX_IO0);
		res1 = find_resource(dev, PNP_IDX_IO1);
		init_pc_keyboard(res0->base, res1->base, &conf->keyboard);
		break;
	}
}

static void it8716f_pnp_set_resources(device_t dev)
{
	pnp_enter_ext_func_mode(dev);  
	pnp_set_resources(dev);
        pnp_exit_ext_func_mode(dev);  
        
}       
        
static void it8716f_pnp_enable_resources(device_t dev)
{       
        pnp_enter_ext_func_mode(dev);  
	pnp_enable_resources(dev);               
        pnp_exit_ext_func_mode(dev);  
}

static void it8716f_pnp_enable(device_t dev)
{

	pnp_enter_ext_func_mode(dev);   
	pnp_set_logical_device(dev);
	pnp_set_enable(dev, dev->enabled);
	pnp_exit_ext_func_mode(dev);  
}

static struct device_operations ops = {
	.read_resources   = pnp_read_resources,
	.set_resources    = it8716f_pnp_set_resources,
	.enable_resources = it8716f_pnp_enable_resources,
	.enable           = it8716f_pnp_enable,
	.init             = it8716f_init,
};

static struct pnp_info pnp_dev_info[] = {
        { &ops, IT8716F_FDC,  PNP_IO0 | PNP_IRQ0 | PNP_DRQ0, { 0x07f8, 0}, },
        { &ops, IT8716F_SP1,  PNP_IO0 | PNP_IRQ0, { 0x7f8, 0 }, },
        { &ops, IT8716F_SP2,  PNP_IO0 | PNP_IRQ0, { 0x7f8, 0 }, },
        { &ops, IT8716F_PP,   PNP_IO0 | PNP_IRQ0 | PNP_DRQ0, { 0x07f8, 0}, },
        { &ops, IT8716F_EC, },
        { &ops, IT8716F_KBCK,  PNP_IO0 | PNP_IO1 | PNP_IRQ0, { 0x7ff, 0 }, { 0x7ff, 0x4}, },
        { &ops, IT8716F_KBCM,  PNP_IRQ0, },
        // No 7 { 0,},
        { &ops, IT8716F_MIDI, PNP_IO0 | PNP_IRQ0, {0x7fe, 0x4}, },
        { &ops, IT8716F_GAME, PNP_IO0, { 0x7ff, 0 },},
        { &ops, IT8716F_CIR, },
};

static void enable_dev(struct device *dev)
{
	pnp_enable_devices(dev, &ops,
		sizeof(pnp_dev_info)/sizeof(pnp_dev_info[0]), pnp_dev_info);
}

struct chip_operations superio_ite_it8716f_ops = {
	CHIP_NAME("ITE it8716f")
	.enable_dev = enable_dev,
};
